@@ -7,7 +7,6 @@ __license__   = "MIT"
 """ Condor job adaptor implementation
 """
 
-import saga.utils.which
 import saga.utils.pty_shell
 
 import saga.adaptors.base
@@ -20,7 +19,8 @@ import re
 import os
 import time
 from copy import deepcopy
-from cgi import parse_qs
+from urlparse import parse_qs
+from tempfile import NamedTemporaryFile
 
 SYNC_CALL = saga.adaptors.cpi.decorators.SYNC_CALL
 ASYNC_CALL = saga.adaptors.cpi.decorators.ASYNC_CALL
@@ -75,6 +75,11 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
     """
     condor_file = str()
 
+    #
+    # HTCondor quoting/escaping:
+    # http://research.cs.wisc.edu/htcondor/manual/current/condor_submit.html#SECTION0012514000000000000000
+    #
+
     ##### OPTIONS PASSED VIA JOB SERVICE URL #####
     ##
     if option_dict is not None:
@@ -82,6 +87,8 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
         # special treatment for universe - defaults to 'vanilla'
         if 'universe' not in option_dict:
             condor_file += "\nuniverse = vanilla"
+
+
         for (key, value) in option_dict.iteritems():
             condor_file += "\n%s = %s" % (key, value)
 
@@ -95,11 +102,11 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
     condor_file += "\nexecutable = /bin/env"
 
     # arguments -> arguments
-    arguments = 'arguments = \\"/bin/sh -c \''
+    arguments = 'arguments = \"/bin/sh -c \''
 
     # The actual executable becomes the first argument.
     if jd.executable is not None:
-        arguments += "%s " % jd.executable
+        arguments += "%s" % jd.executable
 
     if jd.arguments is not None:
 
@@ -116,58 +123,30 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
             arg = arg.replace('"', '""')
     
             # Escape dollars (for environment variables)
-            arg = arg.replace('$', '\\$')
+            #arg = arg.replace('$', '\\$')
     
-            arguments += "%s " % arg
+            arguments += " %s" % arg
 
     # close the quote opened earlier 
-    arguments += '\'\\"'
+    arguments += '\'\"'
 
     condor_file += "\n%s" % arguments
 
-    # file_transfer -> transfer_input_files
-    if jd.file_transfer is not None:
-        td = TransferDirectives(jd.file_transfer)
+    # Transfer Directives
+    if jd.attribute_exists('transfer_directives'):
+        # All checking is done in _handle_file_transfers() already.
+        td = jd.transfer_directives
 
-        if len(td.in_append_dict) > 0:
-            raise Exception('FileTransfer append syntax (>>) not supported by Condor: %s' % td.in_append_dict)
-        if len(td.out_append_dict) > 0:
-            raise Exception('FileTransfer append syntax (<<) not supported by Condor: %s' % td.out_append_dict)
-
-        if len(td.in_overwrite_dict) > 0:
+        if hasattr(td, 'transfer_input_files'):
             transfer_input_files = "transfer_input_files = "
-            for (source, target) in td.in_overwrite_dict.iteritems():
-                # make sure source is file an not dir
-                (s_path, s_entry) = os.path.split(source)
-                if len(s_entry) < 1:
-                    raise Exception('Condor accepts only files (not directories) as FileTransfer sources: %s' % source)
-                # make sure target is just a file
-                (t_path, t_entry) = os.path.split(target)
-                if len(t_path) > 1:
-                    raise Exception('Condor accepts only filenames (without paths) as FileTransfer targets: %s' % target)
-                # make sure source and target file are the same
-                if s_entry != t_entry:
-                    raise Exception('For Condor source file name and target file name have to be identical: %s != %s' % (s_entry, t_entry))
-                # entry ok - add to job script
+            for source in td.transfer_input_files:
                 transfer_input_files += "%s, " % source
             condor_file += "\n%s" % transfer_input_files
 
-        if len(td.out_overwrite_dict) > 0:
+        if hasattr(td, 'transfer_output_files'):
             transfer_output_files = "transfer_output_files = "
-            for (source, target) in td.out_overwrite_dict.iteritems():
-                # make sure source is file an not dir
-                (s_path, s_entry) = os.path.split(source)
-                if len(s_entry) < 1:
-                    raise Exception('Condor accepts only files (not directories) as FileTransfer sources: %s' % source)
-                # make sure target is just a file
-                (t_path, t_entry) = os.path.split(target)
-                if len(t_path) > 1:
-                    raise Exception('Condor accepts only filenames (without paths) as FileTransfer targets: %s' % target)
-                # make sure source and target file are the same
-                if s_entry != t_entry:
-                    raise Exception('For Condor source file name and target file name have to be identical: %s != %s' % (s_entry, t_entry))
-                # entry ok - add to job script
-                transfer_output_files += "%s, " % source
+            for target in td.transfer_output_files:
+                transfer_output_files += "%s, " % target
             condor_file += "\n%s" % transfer_output_files
 
     # always define log. if 'jd.output' is defined, we use it 
@@ -193,12 +172,13 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
         condor_file += "\nerror = %s " % jd.error 
 
     # environment -> environment
-    environment = "environment = "
+    # http://research.cs.wisc.edu/htcondor/manual/current/condor_submit.html#SECTION0012514000000000000000
+    environment = "environment ="
     if jd.environment is not None:
         variable_list = str()
         for key in jd.environment.keys(): 
-            variable_list += "%s=%s;" % (key, jd.environment[key])
-        environment += "%s " % variable_list
+            variable_list += "%s=%s " % (key, jd.environment[key])
+        environment += " \"%s\"" % variable_list.strip()
     condor_file += "\n%s" % environment
 
     # project -> +ProjectName
@@ -215,7 +195,8 @@ def _condorscript_generator(url, logger, jd, option_dict=None):
         condor_file += "\n%s" % sitelist
         condor_file += "\n%s" % requirements
 
-    condor_file += "\n\nqueue"
+    condor_file += "\n\nqueue\n"
+    condor_file += "\n##### END OF FILE #####\n"
 
     return condor_file
 
@@ -275,7 +256,7 @@ The (HT)Condor(-G) adaptor allows to run and manage jobs on a
 """,
     "example": "examples/jobs/condorjob.py",
     "schemas": {"condor"        : "connect to a local gateway",
-                "condor+ssh"    : "conenct to a remote gateway via SSH",
+                "condor+ssh"    : "connect to a remote gateway via SSH",
                 "condor+gsissh ": "connect to a remote gateway via GSISSH"}
 }
 
@@ -283,8 +264,8 @@ The (HT)Condor(-G) adaptor allows to run and manage jobs on a
 # the adaptor info is used to register the adaptor with SAGA
 #
 _ADAPTOR_INFO = {
-    "name"             :    _ADAPTOR_NAME,
-    "version"          : "v0.1",
+    "name"             : _ADAPTOR_NAME,
+    "version"          : "v0.2",
     "schemas"          : _ADAPTOR_SCHEMAS,
     "capabilities"     : _ADAPTOR_CAPABILITIES,
     "cpis": [
@@ -312,11 +293,10 @@ class Adaptor (saga.adaptors.base.Base):
     #
     def __init__(self):
 
-        saga.adaptors.base.Base.__init__(self,
-            _ADAPTOR_INFO, _ADAPTOR_OPTIONS)
+        saga.adaptors.base.Base.__init__(self, _ADAPTOR_INFO, _ADAPTOR_OPTIONS)
 
         self.id_re = re.compile('^\[(.*)\]-\[(.*?)\]$')
-        self.opts = self.get_config()
+        self.opts  = self.get_config (_ADAPTOR_NAME)
 
     # ----------------------------------------------------------------
     #
@@ -380,9 +360,9 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
             for key, val in parse_qs(rm_url.query).iteritems():
                 self.query_options[key] = val[0]
 
-        # we need to extrac the scheme for PTYShell. That's basically the
-        # job.Serivce Url withou the condor+ part. We use the PTYShell to execute
-        # condor commands either locally or via gsissh or ssh.
+        # we need to extract the scheme for PTYShell. That's basically the
+        # job.Service Url without the condor+ part. We use the PTYShell to
+        # execute condor commands either locally or via gsissh or ssh.
         if rm_scheme == "condor":
             pty_url.scheme = "fork"
         elif rm_scheme == "condor+ssh":
@@ -392,7 +372,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
 
         # these are the commands that we need in order to interact with Condor.
         # the adaptor will try to find them during initialize(self) and bail
-        # out in case they are note avaialbe.
+        # out in case they are not available.
         self._commands = {'condor_version': None,
                           'condor_submit':  None,
                           'condor_q':       None,
@@ -457,13 +437,60 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
     def _job_run(self, jd):
         """ runs a job via qsub
         """
+
+        # Because we do funky shit with env and sh, we need to explicitly add
+        # the executable to the transfer list.
+        # (Of course not if the executable is already on the target systems,
+        # defined by the fact that it starts with ./
+        if jd.executable.startswith('./') and os.path.exists(jd.executable):
+
+            # TODO: Check if the executable is already in the file_transfer list,
+            # because then we don't need to implicitly add it anymore.
+            # (For example, if the executable is not in the local directory, it needs
+            # to be explicitly added.)
+
+            exe = jd.executable[len('./'):]
+            exe_transfer = '%s > %s' % (exe, exe)
+
+            if jd.file_transfer:
+                jd.file_transfer.append(exe_transfer)
+            else:
+                jd.file_transfer = [ exe_transfer ]
+
+        if jd.file_transfer is not None:
+            jd.transfer_directives = TransferDirectives(jd.file_transfer)
+
+            self._handle_file_transfers(jd)
+
+            td = jd.transfer_directives
+            if not (hasattr(td, 'transfer_output_files') or hasattr(td, 'transfer_input_files')):
+                raise RuntimeError('One of them should be set!')
+
         # create a Condor job script from SAGA job description
-        script = _condorscript_generator(url=self.rm, logger=self._logger, jd=jd,
-            option_dict=self.query_options)
+        script = _condorscript_generator(url=self.rm, logger=self._logger,
+                jd=jd, option_dict=self.query_options)
         self._logger.info("Generated Condor script: %s" % script)
 
-        ret, out, _ = self.shell.run_sync('echo "%s" | %s -' \
-            % (script, self._commands['condor_submit']['path']))
+        submit_file = NamedTemporaryFile(mode='w', suffix='.condor',
+                    prefix='tmp-saga-', delete=False)
+        submit_file.write(script)
+        submit_file.close()
+        self._logger.info("Written Condor script locally: %s" % submit_file.name)
+
+        if self.shell.url.scheme == "ssh":
+            self._logger.info("Transferring Condor script to: %s" % self.shell.url)
+            submit_file_name = os.path.basename(submit_file.name)
+            self.shell.stage_to_remote(submit_file.name, submit_file_name, cp_flags="-P")
+
+        elif self.shell.url.scheme == "gsissh":
+            raise NotImplemented("GSISSH support for Condor not implemented.")
+        else:
+            submit_file_name = submit_file.name
+
+
+        ret, out, _ = self.shell.run_sync('%s -verbose %s' \
+            % (self._commands['condor_submit']['path'], submit_file_name))
+
 
         if ret != 0:
             # something went wrong
@@ -494,9 +521,20 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
                 'create_time':  None,
                 'start_time':   None,
                 'end_time':     None,
-                'gone':         False
+                'gone':         False,
+                'transfers':    None,
+                'stdout':       None,
+                'stderr':       None
             }
 
+            # remove submit file(s)
+            # XXX: maybe leave them in case of debugging?
+            if self.shell.url.scheme == 'ssh':
+                ret, out, _ = self.shell.run_sync ('rm %s' % submit_file_name)
+            elif self.shell.url.scheme == 'gsissh':
+                raise NotImplemented("GSISSH support for Condor not implemented.")
+            os.remove(submit_file.name)
+
             return job_id
 
     # ----------------------------------------------------------------
@@ -524,7 +562,10 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
                 'create_time':  None,
                 'start_time':   None,
                 'end_time':     None,
-                'gone':         False
+                'gone':         False,
+                'transfers':    None,
+                'stdout':       None,
+                'stderr':       None
             }
 
             results = out.split('\n')
@@ -543,6 +584,59 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
 
             return job_info
 
+    def _handle_file_transfers(self, jd):
+
+        td = jd.transfer_directives
+
+        # Condor specific safety checks
+        if len(td.in_append_dict) > 0:
+            raise Exception('FileTransfer append syntax (>>) not supported by Condor: %s' % td.in_append_dict)
+        if len(td.out_append_dict) > 0:
+            raise Exception('FileTransfer append syntax (<<) not supported by Condor: %s' % td.out_append_dict)
+
+        if len(td.in_overwrite_dict) > 0:
+            td.transfer_input_files = []
+            for (source, target) in td.in_overwrite_dict.iteritems():
+                # make sure source is file an not dir
+                (s_path, s_entry) = os.path.split(source)
+                if len(s_entry) < 1:
+                    raise Exception('Condor accepts only files (not directories) as FileTransfer sources: %s' % source)
+                    # make sure target is just a file
+                (t_path, t_entry) = os.path.split(target)
+                if len(t_path) > 1:
+                    raise Exception('Condor accepts only filenames (without paths) as FileTransfer targets: %s' % target)
+                    # make sure source and target file are the same
+                if s_entry != t_entry:
+                    raise Exception('For Condor source file name and target file name have to be identical: %s != %s' % (s_entry, t_entry))
+                    # entry ok - add to job script
+
+                # add for later use by job script generator
+                td.transfer_input_files.append(target)
+
+                if self.shell.url.scheme == "ssh":
+                    self._logger.info("Transferring file %s to %s" % (source, target))
+                    self.shell.stage_to_remote(source, target)
+
+        if len(td.out_overwrite_dict) > 0:
+            td.transfer_output_files = []
+            for (source, target) in td.out_overwrite_dict.iteritems():
+                # make sure source is file an not dir
+                (s_path, s_entry) = os.path.split(source)
+                if len(s_entry) < 1:
+                    raise Exception('Condor accepts only files (not directories) as FileTransfer sources: %s' % source)
+                    # make sure target is just a file
+                (t_path, t_entry) = os.path.split(target)
+                if len(t_path) > 1:
+                    raise Exception('Condor accepts only filenames (without paths) as FileTransfer targets: %s' % target)
+                    # make sure source and target file are the same
+                if s_entry != t_entry:
+                    raise Exception('For Condor source file name and target file name have to be identical: %s != %s' % (s_entry, t_entry))
+                    # entry ok - add to job script
+
+                # add for later use by job script generator
+                td.transfer_output_files.append(target)
+
+
     # ----------------------------------------------------------------
     #
     def _job_get_info(self, job_id):
@@ -551,7 +645,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
 
         # if we don't have the job in our dictionary, we don't want it
         if job_id not in self.jobs:
-            message = "Unkown job ID: %s. Can't update state." % job_id
+            message = "Unknown job ID: %s. Can't update state." % job_id
             log_error_and_raise(message, saga.NoSuccess, self._logger)
 
         # prev. info contains the info collect when _job_get_info
@@ -576,35 +670,44 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
             % (self._commands['condor_q']['path'], pid))
 
         if ret != 0:
-            #if ("Unknown Job Id" in out):
-                # Let's see if the previous job state was runnig or pending. in
-                # that case, the job is gone now, which can either mean DONE,
-                # or FAILED. the only thing we can do is set it to 'DONE'
             if prev_info['state'] in [saga.job.RUNNING, saga.job.PENDING]:
 
                 # run the Condor 'condor_history' command to get info about 
                 # finished jobs
-                ret, out, _ = self.shell.run_sync("%s -long %s | \
-                    egrep '(ExitCode)'" \
+                ret, out, _ = self.shell.run_sync("%s -long -match 1 %s | \
+                    egrep '(ExitCode)|(TransferOutput)|(CompletionDate)|(JobCurrentStartDate)|(QDate)|(Err)|(Out)'" \
                     % (self._commands['condor_history']['path'], pid))
                 
                 if ret != 0:
                     message = "Error getting job history via 'condor_history': %s" % out
                     log_error_and_raise(message, saga.NoSuccess, self._logger)
 
-                # sometimes we get other crap (like biff)
-                out = out.split('\n')[0] 
+                # parse the egrep result. this should look something like this:
+                # ExitCode = 0
+                # TransferOutput = "radical.txt"
+                results = out.split('\n')
+                for result in results:
+                    if len(result.split('=')) == 2:
+                        key, val = result.split('=')
+                        key = key.strip()  # strip() removes whitespaces at the
+                        val = val.strip()  # beginning and the end of the string
 
-                if len(out.split('=')) != 2:
-                    message = "No ExitCode found via 'condor_history'"
-                    log_error_and_raise(message, saga.NoSuccess, self._logger)
+                        if key == 'ExitCode':
+                            curr_info['returncode'] = int(val)
+                        elif key == 'TransferOutput':
+                            curr_info['transfers'] = val
+                        elif key == 'QDate':
+                            curr_info['create_time'] = val
+                        elif key == 'JobCurrentStartDate':
+                            curr_info['start_time'] = val
+                        elif key == 'CompletionDate':
+                            curr_info['end_time'] = val
+                        elif key == 'Out':
+                            curr_info['stdout'] = val
+                        elif key == 'Err':
+                            curr_info['stderr'] = val
 
-                _, val = out.split('=')
-                retcode = int(val.strip())
-                    
-                curr_info['returncode'] = retcode
-
-                if retcode == 0:
+                if curr_info['returncode'] == 0:
                     curr_info['state'] = saga.job.DONE
                 else:
                     curr_info['state'] = saga.job.FAILED
@@ -636,6 +739,48 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
                     elif key == 'CompletionDate':
                         curr_info['end_time'] = val
 
+        if curr_info['gone'] is True:
+            # If we are running over SSH, copy the output to our local system
+            if self.shell.url.scheme == "ssh":
+                files = []
+
+                if curr_info['transfers']:
+                    t = curr_info['transfers']
+                    self._logger.debug("TransferOutput: %s" % t)
+
+                    # Remove leading and ending double quotes
+                    if t.startswith('"') and t.endswith('"'):
+                        t = t[1:-1]
+
+                    # Parse comma separated list
+                    files += t.split(',')
+
+                if curr_info['stdout']:
+                    t = curr_info['stdout']
+                    self._logger.debug("StdOut: %s" % t)
+
+                    # Remove leading and ending double quotes
+                    if t.startswith('"') and t.endswith('"'):
+                        t = t[1:-1]
+
+                    files.append(t)
+
+                if curr_info['stderr']:
+                    t = curr_info['stderr']
+                    self._logger.debug("StdErr: %s" % t)
+
+                    # Remove leading and ending double quotes
+                    if t.startswith('"') and t.endswith('"'):
+                        t = t[1:-1]
+
+                    files.append(t)
+
+                # Transfer list of files
+                for f in files:
+                    f = f.strip()
+                    self._logger.info("Transferring file %s" % f)
+                    self.shell.stage_from_remote(f, f)
+
         # return the new job info dict
         return curr_info
 
@@ -734,7 +879,7 @@ class CondorJobService (saga.adaptors.cpi.job.Service):
             message = "Error canceling job via 'condor_rm': %s" % out
             log_error_and_raise(message, saga.NoSuccess, self._logger)
 
-        # assume the job was succesfully canceld
+        # assume the job was successfully canceled
         self.jobs[job_id]['state'] = saga.job.CANCELED
 
     # ----------------------------------------------------------------
@@ -903,7 +1048,7 @@ class CondorJob (saga.adaptors.cpi.job.Job):
     #
     @SYNC_CALL
     def get_state(self):
-        """ mplements saga.adaptors.cpi.job.Job.get_state()
+        """ implements saga.adaptors.cpi.job.Job.get_state()
         """
         if self._started is False:
             # jobs that are not started are always in 'NEW' state
