@@ -217,7 +217,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
 
         if  sumisc.url_is_compatible (cwdurl, tgt) :
 
-            ret, out, _ = self.shell.run_sync ("mkdir -p %s\n" % (dirname))
+            ret, out, _ = self.shell.run_sync (" mkdir -p '%s'\n" % (dirname))
             if  ret != 0 :
                 raise saga.NoSuccess ("failed at mkdir '%s': (%s) (%s)" \
                                    % (dirname, ret, out))
@@ -228,7 +228,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
                 raise saga.BadParameter ("schema of mkdir target is not supported (%s)" \
                                       % (tgt))
 
-            ret, out, _ = self.local.run_sync ("mkdir -p %s\n" % (dirname))
+            ret, out, _ = self.local.run_sync (" mkdir -p '%s'\n" % (dirname))
             if  ret != 0 :
                 raise saga.NoSuccess ("failed at mkdir '%s': (%s) (%s)" \
                                    % (dirname, ret, out))
@@ -259,7 +259,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self.cwdurl.path = self.cwd
 
         self.shell       = sups.PTYShell     (self.url, self.session, self._logger)
-        self._copy_shell = None
+        self.copy_shell  = None
 
       # self.shell.set_initialize_hook (self.initialize)
       # self.shell.set_finalize_hook   (self.finalize)
@@ -280,11 +280,11 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
     #
     def _get_copy_shell (self, tgt) :
 
-        if  not self._copy_shell :
+        if  not self.copy_shell :
             self._logger.debug ("new copy shell (%s)" % (tgt))
-            self._copy_shell = sups.PTYShell (tgt, self.session, self._logger)
+            self.copy_shell = sups.PTYShell (tgt, self.session, self._logger)
 
-        return self._copy_shell
+        return self.copy_shell
 
 
     # ----------------------------------------------------------------
@@ -297,11 +297,11 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cmd = ""
 
         if  self.flags & saga.filesystem.CREATE_PARENTS :
-            cmd = "mkdir -p %s ;  cd %s" % (self.url.path, self.url.path)
+            cmd = " mkdir -p '%s' ;  cd '%s'" % (self.url.path, self.url.path)
         elif self.flags & saga.filesystem.CREATE :
-            cmd = "mkdir    %s ;  cd %s" % (self.url.path, self.url.path)
+            cmd = " mkdir    '%s' ;  cd '%s'" % (self.url.path, self.url.path)
         else :
-            cmd = "test -d  %s && cd %s" % (self.url.path, self.url.path)
+            cmd = " test -d  '%s' && cd '%s'" % (self.url.path, self.url.path)
 
         ret, out, _ = self.shell.run_sync (cmd)
 
@@ -320,6 +320,15 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         if  kill and self.shell :
             self.shell.finalize (True)
             self.shell = None
+
+        if  kill and self.local :
+            self.local.finalize (True)
+            self.local = None
+
+        if  kill and self.copy_shell :
+            self.copy_shell.finalize (True)
+            self.copy_shell = None
+
         self.valid = False
 
 
@@ -386,11 +395,11 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
 
 
         if  flags & saga.filesystem.CREATE_PARENTS :
-            cmd = "mkdir -p %s ;  cd %s" % (tgturl.path, tgturl.path)
+            cmd = " mkdir -p '%s' ;  cd '%s'" % (tgturl.path, tgturl.path)
         elif flags & saga.filesystem.CREATE :
-            cmd = "mkdir    %s ;  cd %s" % (tgturl.path, tgturl.path)
+            cmd = " mkdir    '%s' ;  cd '%s'" % (tgturl.path, tgturl.path)
         else :
-            cmd = "test -d  %s && cd %s" % (tgturl.path, tgturl.path)
+            cmd = " test -d  '%s' && cd '%s'" % (tgturl.path, tgturl.path)
 
         ret, out, _ = self.shell.run_sync (cmd)
 
@@ -434,9 +443,9 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         if  None == npat :
             npat = ""
         else :
-            npat = "-d %s" % npat
+            npat = '-d %s' % npat
 
-        ret, out, _ = self.shell.run_sync ("/bin/ls -C1 %s\n" % npat)
+        ret, out, _ = self.shell.run_sync (" /bin/ls -C1 %s\n" % npat)
 
         if  ret != 0 :
             raise saga.NoSuccess ("failed to list(): (%s)(%s)" \
@@ -496,7 +505,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
             sumisc.url_is_compatible (cwdurl, tgt) :
 
             # print "shell cp"
-            ret, out, err = self.shell.run_sync ("cp %s %s %s\n" % (rec_flag, src.path, tgt.path))
+            ret, out, err = self.shell.run_sync (" cp %s '%s' '%s'\n" % (rec_flag, src.path, tgt.path))
             if  ret != 0 :
                 raise saga.NoSuccess ("copy (%s -> %s) failed (%s): (out: %s) (err: %s)" \
                                    % (src, tgt, ret, out, err))
@@ -609,7 +618,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
             sumisc.url_is_compatible (cwdurl, tgt) :
 
             # print "shell ln"
-            ret, out, err = self.shell.run_sync ("ln -s %s %s\n" % (src.path, tgt.path))
+            ret, out, err = self.shell.run_sync (" ln -s '%s' '%s'\n" % (src.path, tgt.path))
             if  ret != 0 :
                 raise saga.NoSuccess ("link (%s -> %s) failed (%s): (out: %s) (err: %s)" \
                                    % (src, tgt, ret, out, err))
@@ -680,7 +689,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
 
         if  sumisc.url_is_compatible (cwdurl, tgt) :
 
-            ret, out, err = self.shell.run_sync ("rm -f %s %s\n" % (rec_flag, tgt.path))
+            ret, out, err = self.shell.run_sync (" rm -f %s '%s'\n" % (rec_flag, tgt.path))
             if  ret != 0 :
                 raise saga.NoSuccess ("remove (%s) failed (%s): (out: %s) (err: %s)" \
                                    % (tgt, ret, out, err))
@@ -704,7 +713,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         if  flags & saga.filesystem.EXCLUSIVE : 
             # FIXME: this creates a race condition between testing for exclusive
             # mkdir and creating the dir.
-            ret, out, _ = self.shell.run_sync ("test -d %s " % tgt.path)
+            ret, out, _ = self.shell.run_sync (" test -d '%s' " % tgt.path)
 
             if  ret != 0 :
                 raise saga.AlreadyExists ("make_dir target (%s) exists (%s)" \
@@ -716,7 +725,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         if  flags & saga.filesystem.CREATE_PARENTS : 
             options += "-p"
 
-        self.shell.run_sync ("mkdir %s %s" % (options, tgt.path))
+        self.shell.run_sync (" mkdir %s '%s'" % (options, tgt.path))
 
    
     # ----------------------------------------------------------------
@@ -738,7 +747,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        ret, out, _ = self.shell.run_sync ("du -ks %s  | xargs | cut -f 1 -d ' '\n" % tgt.path)
+        ret, out, _ = self.shell.run_sync (" du -ks '%s'  | xargs | cut -f 1 -d ' '\n" % tgt.path)
         if  ret != 0 :
             raise saga.NoSuccess ("get size for (%s) failed (%s): (%s)" \
                                % (tgt, ret, out))
@@ -772,7 +781,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -d %s && test ! -h %s" % (tgt.path, tgt.path))
+        ret, out, _ = self.shell.run_sync (" test -d '%s' && test ! -h '%s'" % (tgt.path, tgt.path))
 
         return True if ret == 0 else False
    
@@ -797,7 +806,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -f %s && test ! -h %s" % (tgt.path, tgt.path))
+        ret, out, _ = self.shell.run_sync (" test -f '%s' && test ! -h '%s'" % (tgt.path, tgt.path))
 
         return True if ret == 0 else False
    
@@ -822,7 +831,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -h %s" % tgt.path)
+        ret, out, _ = self.shell.run_sync (" test -h '%s'" % tgt.path)
 
         return True if ret == 0 else False
    
@@ -855,7 +864,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
         _cpi_base = super  (ShellFile, self)
         _cpi_base.__init__ (api, adaptor)
 
-        self._copy_shell = None
+        self.copy_shell = None
 
 
     # ----------------------------------------------------------------
@@ -880,7 +889,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         if  sumisc.url_is_compatible (cwdurl, tgt) :
 
-            ret, out, _ = self.shell.run_sync ("mkdir -p %s\n" % (dirname))
+            ret, out, _ = self.shell.run_sync (" mkdir -p '%s'\n" % (dirname))
             if  ret != 0 :
                 raise saga.NoSuccess ("failed at mkdir '%s': (%s) (%s)" \
                                    % (dirname, ret, out))
@@ -891,7 +900,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
                 raise saga.BadParameter ("schema of mkdir target is not supported (%s)" \
                                       % (tgt))
 
-            ret, out, _ = self.local.run_sync ("mkdir -p %s\n" % (dirname))
+            ret, out, _ = self.local.run_sync (" mkdir -p '%s'\n" % (dirname))
             if  ret != 0 :
                 raise saga.NoSuccess ("failed at mkdir '%s': (%s) (%s)" \
                                    % (dirname, ret, out))
@@ -906,10 +915,10 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
     #
     def _get_copy_shell (self, tgt) :
 
-        if  not self._copy_shell :
-            self._copy_shell = sups.PTYShell (tgt, self.session, self._logger)
+        if  not self.copy_shell :
+            self.copy_shell = sups.PTYShell (tgt, self.session, self._logger)
 
-        return self._copy_shell
+        return self.copy_shell
 
 
     # ----------------------------------------------------------------
@@ -981,22 +990,22 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
         dirname = sumisc.url_get_dirname  (self.url)
 
         if  self.flags & saga.filesystem.CREATE_PARENTS :
-            cmd = "mkdir -p %s; touch %s" % (dirname, self.url.path)
-            self._logger.info ("mkdir %s; touch %s" % (dirname, self.url.path))
+            cmd = " mkdir -p '%s'; touch '%s'" % (dirname, self.url.path)
+            self._logger.info ("mkdir '%s'; touch '%s'" % (dirname, self.url.path))
 
         elif self.flags & saga.filesystem.CREATE :
-            cmd = "touch %s" % (self.url.path)
+            cmd = " touch '%s'" % (self.url.path)
             self._logger.info ("touch %s" % self.url.path)
 
         else :
-            cmd = "true"
+            cmd = " true"
 
 
         if  self.flags & saga.filesystem.READ :
-            cmd += "; test -r %s" % (self.url.path)
+            cmd += "; test -r '%s'" % (self.url.path)
 
         if  self.flags & saga.filesystem.WRITE :
-            cmd += "; test -w %s" % (self.url.path)
+            cmd += "; test -w '%s'" % (self.url.path)
 
         ret, out, _ = self.shell.run_sync (cmd)
 
@@ -1006,7 +1015,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
             elif self.flags & saga.filesystem.CREATE :
                 raise saga.BadParameter ("cannot open/create: '%s' - %s" % (self.url.path, out))
             else :
-                raise saga.DoesNotExist("file does not exist: '%s' - %s" % (self.url.path, out))
+                raise saga.DoesNotExist("File does not exist: '%s' - %s" % (self.url.path, out))
 
         self._logger.info ("file initialized (%s)(%s)" % (ret, out))
 
@@ -1025,6 +1034,10 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
             self.local.finalize (True)
             self.local = None
 
+        if  kill and self.copy_shell :
+            self.copy_shell.finalize (True)
+            self.copy_shell = None
+
         self.valid = False
 
 
@@ -1080,7 +1093,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
             sumisc.url_is_compatible (cwdurl, tgt) :
 
             # print "shell cp"
-            ret, out, _ = self.shell.run_sync ("cp %s %s %s\n" % (rec_flag, src.path, tgt.path))
+            ret, out, _ = self.shell.run_sync (" cp %s '%s' '%s'\n" % (rec_flag, src.path, tgt.path))
             if  ret != 0 :
                 raise saga.NoSuccess ("copy (%s -> %s) failed (%s): (%s)" \
                                    % (src, tgt, ret, out))
@@ -1177,7 +1190,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
             sumisc.url_is_compatible (cwdurl, tgt) :
 
             # print "shell ln"
-            ret, out, err = self.shell.run_sync ("ln -s %s %s\n" % (src.path, tgt.path))
+            ret, out, err = self.shell.run_sync (" ln -s '%s' '%s'\n" % (src.path, tgt.path))
             if  ret != 0 :
                 raise saga.NoSuccess ("link (%s -> %s) failed (%s): (out: %s) (err: %s)" \
                                    % (src, tgt, ret, out, err))
@@ -1272,7 +1285,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
         if  flags & saga.filesystem.RECURSIVE : 
             rec_flag  += "-r "
 
-        ret, out, _ = self.shell.run_sync ("rm -f %s %s\n" % (rec_flag, tgt.path))
+        ret, out, _ = self.shell.run_sync (" rm -f %s '%s'\n" % (rec_flag, tgt.path))
         if  ret != 0 :
             raise saga.NoSuccess ("remove (%s) failed (%s): (%s)" \
                                % (tgt, ret, out))
@@ -1291,10 +1304,10 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         if  self.is_dir_self () :
             size_mult   = 1024   # see '-k' option to 'du'
-            ret, out, _ = self.shell.run_sync ("du -ks %s  | xargs | cut -f 1 -d ' '\n" \
+            ret, out, _ = self.shell.run_sync (" du -ks '%s'  | xargs | cut -f 1 -d ' '\n" \
                                             % self.url.path)
         else :
-            ret, out, _ = self.shell.run_sync ("wc -c %s | xargs | cut -f 1 -d ' '\n" \
+            ret, out, _ = self.shell.run_sync (" wc -c '%s' | xargs | cut -f 1 -d ' '\n" \
                                             % self.url.path)
 
         if  ret != 0 :
@@ -1319,7 +1332,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         cwdurl = saga.Url (self.url) # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -d %s && test ! -h %s" % (cwdurl.path, cwdurl.path))
+        ret, out, _ = self.shell.run_sync (" test -d '%s' && test ! -h '%s'" % (cwdurl.path, cwdurl.path))
 
         return True if ret == 0 else False
 
@@ -1333,7 +1346,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         cwdurl = saga.Url (self.url) # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -f %s && test ! -h %s" % (cwdurl.path, cwdurl.path))
+        ret, out, _ = self.shell.run_sync (" test -f '%s' && test ! -h '%s'" % (cwdurl.path, cwdurl.path))
 
         return True if ret == 0 else False
    
@@ -1347,7 +1360,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         cwdurl = saga.Url (self.url) # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -h %s" % cwdurl.path)
+        ret, out, _ = self.shell.run_sync (" test -h '%s'" % cwdurl.path)
 
         return True if ret == 0 else False
    
@@ -1361,7 +1374,7 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
 
         cwdurl = saga.Url (self.url) # deep copy
 
-        ret, out, _ = self.shell.run_sync ("test -f %s && test ! -h %s" % (cwdurl.path, cwdurl.path))
+        ret, out, _ = self.shell.run_sync (" test -f '%s' && test ! -h '%s'" % (cwdurl.path, cwdurl.path))
 
         return True if ret == 0 else False
    
