@@ -1,5 +1,5 @@
 
-__author__    = "Andre Merzky, Ashley Z, Ole Weidner"
+__author__    = "Andre Merzky, Ole Weidner"
 __copyright__ = "Copyright 2012-2013, The SAGA Project"
 __license__   = "MIT"
 
@@ -28,9 +28,11 @@ adaptor_registry = [
                     "saga.adaptors.redis.redis_advert",
                     "saga.adaptors.sge.sgejob",
                     "saga.adaptors.pbs.pbsjob",
+                    "saga.adaptors.lsf.lsfjob",
                     "saga.adaptors.irods.irods_replica",
                     "saga.adaptors.condor.condorjob",
                     "saga.adaptors.slurm.slurm_job",
                     "saga.adaptors.http.http_file",
-                    "saga.adaptors.aws.ec2_resource"
+                    "saga.adaptors.aws.ec2_resource",
+                    "saga.adaptors.loadl.loadljob"
                    ]
