@@ -99,8 +99,6 @@ class PTYProcess (object) :
         :param logger:  logger stream to send status messages to.
         """
 
-        self._debug = False
-
         self.logger = logger
         if  not  self.logger : self.logger = rul.getLogger ('saga', 'PTYProcess') 
         self.logger.debug ("PTYProcess init %s" % self)
@@ -225,11 +223,18 @@ class PTYProcess (object) :
                             wpid, wstat = os.waitpid (self.child, os.WNOHANG)
 
                         except OSError as e :
+
                             # this should not have failed -- child disappeared?
-                            self.exit_code   = None 
-                            self.exit_signal = None
-                            wstat            = None
-                            break
+                            if e.errno == errno.ECHILD :
+                                self.exit_code   = None 
+                                self.exit_signal = None
+                                wstat            = None
+                                break
+                            else :
+                                # other errors are bad, but there is not much to
+                                # be done at this point
+                                self.logger.warning ("ignore waitpid failure on finalize (%s)" % e)
+                                break
 
                         if  wpid :
                             break
@@ -307,6 +312,7 @@ class PTYProcess (object) :
                 # hey, kiddo, whats up?
                 try :
                     wpid, wstat = os.waitpid (self.child, 0)
+
                 except OSError as e :
 
                     if e.errno == errno.ECHILD :
@@ -318,7 +324,7 @@ class PTYProcess (object) :
                         return output
 
                     # no idea what happened -- it is likely bad
-                    raise se.NoSuccess ("waitpid failed")
+                    raise se.NoSuccess ("waitpid failed on wait")
 
 
                 # did we get a note about child termination?
@@ -370,11 +376,23 @@ class PTYProcess (object) :
             # do we have a child which we can check?
             if  self.child :
 
+                wstat = None
+
                 while True :
                   # print 'waitpid %s' % self.child
+                  
                     # hey, kiddo, whats up?
-                    wpid, wstat = os.waitpid (self.child, os.WNOHANG)
-                  # print 'waitpid %s : %s - %s' % (self.child, wpid, wstat)
+                    try :
+                        wpid, wstat = os.waitpid (self.child, os.WNOHANG)
+                      # print 'waitpid %s : %s - %s' % (self.child, wpid, wstat)
+
+                    except OSError as e :
+
+                        if e.errno == errno.ECHILD :
+                            # child disappeared, go to zombie cleanup routine
+                            break
+
+                        raise ("waitpid failed on wait (%s)" % e)
 
                     # did we get a note about child termination?
                     if 0 == wpid :
@@ -627,8 +645,17 @@ class PTYProcess (object) :
         read buffers, this method can be expensive.  
 
         Note: the returned data get '\\\\r' stripped.
+
+        Note: ansi-escape sequences are also stripped before matching, but are
+        kept in the returned data.
         """
 
+        def escape (txt) :
+            pat = re.compile(r'\x1b[^m]*m')
+            return pat.sub ('', txt)
+
+        _debug = False
+
         with self.rlock :
 
             try :
@@ -649,43 +676,49 @@ class PTYProcess (object) :
                 # a pattern, or timeout passes
                 while True :
 
+
+
                   # time.sleep (0.1)
 
                     # skip non-lines
                     if  not data :
                         data += self.read (timeout=_POLLDELAY)
-                  # if  self._debug : print ">>%s<<" % data
+                    if  _debug : print ">>%s<<" % data
 
                     # check current data for any matching pattern
                     for n in range (0, len(patts)) :
 
-                        match = patts[n].search (data)
-                     #  if self._debug : print "==%s==" % patterns[n]
+                        escaped = escape (data)
+                      # print '-- 1 --%s--' % data
+                      # print '-- 2 --%s--' % escaped
+
+                        match = patts[n].search (escaped)
+                        if _debug : print "==%s==" % patterns[n]
 
                         if match :
                             # a pattern matched the current data: return a tuple of
                             # pattern index and matching data.  The remainder of the
                             # data is cached.
-                            ret  = data[0:match.end()]
-                            self.cache = data[match.end():] 
+                            ret  = escaped[0:match.end()]
+                            self.cache = escaped[match.end():] 
 
-                     #      if self._debug : print "~~match!~~ %s" % data[match.start():match.end()]
-                     #      if self._debug : print "~~match!~~ %s" % (len(data))
-                     #      if self._debug : print "~~match!~~ %s" % (str(match.span()))
-                     #      if self._debug : print "~~match!~~ %s" % (ret)
+                            if _debug : print "~~match!~~ %s" % escaped[match.start():match.end()]
+                            if _debug : print "~~match!~~ %s" % (len(escaped))
+                            if _debug : print "~~match!~~ %s" % (str(match.span()))
+                            if _debug : print "~~match!~~ %s" % (ret)
 
                             return (n, ret.replace('\r', ''))
 
                     # if a timeout is given, and actually passed, return
                     # a non-match and a copy of the data we looked at
                     if timeout == 0 :
-                        return (None, str(data))
+                        return (None, str(escaped))
 
                     if timeout > 0 :
                         now = time.time ()
                         if (now-start) > timeout :
-                            self.cache = data
-                            return (None, str(data))
+                            self.cache = escaped
+                            return (None, str(escaped))
 
                     # no match yet, still time -- read more data
                     data += self.read (timeout=_POLLDELAY)
