@@ -80,8 +80,8 @@ _SCRIPTS = {
         'shell'         : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
         'copy_to'       : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
         'copy_from'     : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
-        'copy_to_in'    : "cd ~ && %(cp_exe)s -v %(cp_flags)s %(src)s %(tgt)s",
-        'copy_from_in'  : "cd ~ && %(cp_exe)s -v %(cp_flags)s %(src)s %(tgt)s",
+        'copy_to_in'    : "cd ~ && %(cp_exe)s -v %(cp_flags)s '%(src)s' '%(tgt)s'",
+        'copy_from_in'  : "cd ~ && %(cp_exe)s -v %(cp_flags)s '%(src)s' '%(tgt)s'",
     }
 }
 
@@ -221,13 +221,14 @@ class PTYShellFactory (object) :
             try :
                 prompt_patterns = ["[Pp]assword:\s*$",             # password   prompt
                                    "Enter passphrase for .*:\s*$", # passphrase prompt
+                                   "Token_Response.*:\s*$",        # passtoken  prompt
                                    "want to continue connecting",  # hostkey confirmation
                                    ".*HELLO_\\d+_SAGA$",           # prompt detection helper
-                                   "^(.*[\$#%>\]])\s*$"]             # greedy native shell prompt 
+                                   "^(.*[\$#%>\]])\s*$"]           # greedy native shell prompt 
 
                 # find a prompt
                 # use a very aggressive, but portable prompt setting scheme
-                pty_shell.write ("export PS1='>' >& /dev/null || set prompt='>'\n")
+              # pty_shell.write (" export PS1='$' > /dev/null 2>&1 || set prompt='$'\n")
                 n, match = pty_shell.find (prompt_patterns, delay)
 
                 # this loop will run until we finally find the shell prompt, or
@@ -265,8 +266,8 @@ class PTYShellFactory (object) :
 
                         if  is_shell :
                             # use a very aggressive, but portable prompt setting scheme
-                            pty_shell.write ("export PS1='>' >& /dev/null || set prompt='>'\n")
-                            pty_shell.write ("printf 'HELLO_%%d_SAGA\\n' %d\n" % retries)
+                            pty_shell.write (" export PS1='$' > /dev/null 2>&1 || set prompt='$'\n")
+                            pty_shell.write (" printf 'HELLO_%%d_SAGA\\n' %d\n" % retries)
                             used_trigger = True
 
                         # FIXME:  consider better timeout
@@ -306,13 +307,22 @@ class PTYShellFactory (object) :
 
                     # --------------------------------------------------------------
                     elif n == 2 :
+                        logger.info ("got token prompt")
+                        import getpass
+                        token = getpass.getpass ("enter token: ")
+                        pty_shell.write ("%s\n" % token.strip())
+                        n, match = pty_shell.find (prompt_patterns, delay)
+
+
+                    # --------------------------------------------------------------
+                    elif n == 3 :
                         logger.info ("got hostkey prompt")
                         pty_shell.write ("yes\n")
                         n, match = pty_shell.find (prompt_patterns, delay)
 
 
                     # --------------------------------------------------------------
-                    elif n == 3 :
+                    elif n == 4 :
 
                         # one of the trigger commands got through -- we can now
                         # hope to find the prompt (or the next trigger...)
@@ -325,7 +335,7 @@ class PTYShellFactory (object) :
 
 
                     # --------------------------------------------------------------
-                    elif n == 4 :
+                    elif n == 5 :
 
                         logger.debug ("got initial shell prompt (%s) (%s)" %  (n, match))
 
@@ -337,13 +347,26 @@ class PTYShellFactory (object) :
                                 trigger = "HELLO_%d_SAGA" % retries
 
                                 if  not trigger in found_trigger :
+
                                     logger.debug ("waiting for prompt trigger %s: (%s) (%s)" \
                                                % (trigger, n, match))
                                     # but more retries won't help...
                                     retry_trigger = False
-                                    n = None
+                                    attempts      = 0
+                                    n             = None
+
                                     while not n :
-                                        n, match = pty_shell.find (prompt_patterns, delay)
+
+                                        attempts += 1
+                                        n, match  = pty_shell.find (prompt_patterns, delay)
+
+                                        if  not n :
+                                            if  attempts == 1 :
+                                                pty_shell.write (" printf 'HELLO_%%d_SAGA\\n' %d\n" % retries)
+
+                                            if  attempts > 100 :
+                                                raise se.NoSuccess ("Could not detect shell prompt (timeout)")
+
                                     continue
 
 
@@ -360,13 +383,18 @@ class PTYShellFactory (object) :
     #
     def _get_cp_slave (self, s_cmd, info) :
 
-        host = info.get ('host_str', 'any')
-        
-        if  host not in self._cp_slaves :
+        with self.rlock :
+
+            host = info.get ('host_str', 'any')
+
+            if  host in self._cp_slaves :
+                if  self._cp_slaves[host].alive () :
+                    return self._cp_slaves[host]
+            
             self._cp_slaves[host] = supp.PTYProcess (s_cmd, info['logger'])
             self._initialize_pty (self._cp_slaves[host], info)
 
-        return self._cp_slaves[host]
+            return self._cp_slaves[host]
 
     # --------------------------------------------------------------------------
     #
@@ -428,9 +456,8 @@ class PTYShellFactory (object) :
 
 
             _      = cp_slave.write    ("%s%s\n" % (prep, s_in))
-            _, out = cp_slave.find     (['[\$\>\]] *$'], -1)
-            _, out = cp_slave.find     (['[\$\>\]] *$'], 1.0)
-
+            _, out = cp_slave.find     (['[\$\>\]]\s*$'], -1)
+            _, out = cp_slave.find     (['[\$\>\]]\s*$'], 1.0)
 
             # FIXME: we don't really get exit codes from copy
             # if  cp_slave.exit_code != 0 :
@@ -445,8 +472,9 @@ class PTYShellFactory (object) :
             if 'is not a directory' in out :
                 raise se.BadParameter._log (info['logger'], "File copy failed: %s" % str(out))
 
-            if 'not found' in out :
-                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+            if  'sftp' in s_cmd :
+                if 'not found' in out :
+                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
 
 
             # we interpret the first word on the line as name of src file -- we
@@ -502,9 +530,10 @@ class PTYShellFactory (object) :
             cp_slave = self._get_cp_slave (s_cmd, info)
 
             prep = ""
+
             if  'sftp' in s_cmd :
                 # prepare target dirs for recursive copy, if needed
-                cp_slave.write ("ls %s\n" % src)
+                cp_slave.write (" ls %s\n" % src)
                 _, out = cp_slave.find (["^sftp> "], -1)
 
                 src_list = out[1].split ('/n')
@@ -530,8 +559,9 @@ class PTYShellFactory (object) :
             if 'is not a directory' in out :
                 raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
 
-            if 'not found' in out :
-                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+            if  'sftp' in s_cmd :
+                if 'not found' in out :
+                    raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
 
 
             # we run copy with -v, so get a list of files which have been copied
@@ -601,8 +631,8 @@ class PTYShellFactory (object) :
             elif info['schema']  in _SCHEMAS_SH :
                 info['type']     = "sh"
                 info['sh_args']  = "-i"
-                info['sh_env']   = "/usr/bin/env TERM=vt100"
-                info['cp_env']   = "/usr/bin/env TERM=vt100"
+                info['sh_env']   = "/usr/bin/env TERM=vt100 PS1='PROMPT-$?->'"
+                info['cp_env']   = "/usr/bin/env TERM=vt100 PS1='PROMPT-$?->'"
                 info['fs_root']  = "/"
 
                 if  "SHELL" in os.environ :
@@ -636,6 +666,8 @@ class PTYShellFactory (object) :
                 
             if  info['type'] == "sh" :
 
+                info['sh_env'] = "/usr/bin/env TERM=vt100 "  # avoid ansi escapes
+
                 if not sumisc.host_is_local (url.host) :
                     raise se.BadParameter._log (self.logger, \
                             "expect local host for '%s://', not '%s'" % (url.schema, url.host))
@@ -738,6 +770,3 @@ class PTYShellFactory (object) :
             # registration
             return info
 
-
-
-
